local FOLDER_NAME, private = ...
local _G = _G
local LS = _G.LibStub
local core = LS("AceAddon-3.0"):GetAddon(FOLDER_NAME)
local L = LS("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local LSM = LS("LibSharedMedia-3.0")
local build = GetAddOnMetadata(FOLDER_NAME, "X-BuildDate")
local CloseDropDownMenus, ReloadUI, AceGUIWidgetLSMlists = CloseDropDownMenus, ReloadUI, AceGUIWidgetLSMlists
local pairs, type, wipe = pairs, type, wipe
local getOpt, setOpt, options, moduleOptions = nil, nil, nil, {}
local outlines = {
	[""] = L["None"],
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
	["MONOCHROMEOUTLINE"] = L["Monochrome Outline"],
}

do
	function getOpt(info)
		local key = info[#info]
		return core.db.profile[key]
	end
	function setOpt(info, value)
		local key = info[#info]
		core.db.profile[key] = value
	end
end

local function getOptions()
	if not options then
		options = {
			type = "group",
			name = FOLDER_NAME,
			args = {
				general = {
					order = 0,
					type = "group",
					name = L["General Settings"],
					get = getOpt,
					set = setOpt,
					args = {
						intro = {
							order = 5,
							type = "header",
							name = L["General Settings"],
						},
						space = {
							order = 10,
							type = "description",
							name = " "
						},
						author = {
							order = 15,
							type = "description",
							name = L["Author: "]..private.author
						},
						version = {
							order = 20,
							type = "description",
							name = L["Version: "]..private.version
						},
						builddate = {
							order = 25,
							type = "description",
							name = L["Build Date: "]..build
						},
						space1 = {
							order = 30,
							type = "description",
							name = " "
						},
						options = {
							type = "group",
							name = L["Right-Click Options"],
							inline = true,
							order = 35,
							args = {
								showxps = {
									type = "toggle",
									order = 38,
									width = "full",
									name = L["Show the tooltip module expansion filters on the Right-Click menu."],
									descStyle = "inline",
									desc = "|cffff0000"..L["Enabling and using these options on any module, will affect all of them."].."|r",
									get = function() return core.db.profile.showxps end,
									set = function(info, value)
										core.db.profile.showxps = value
										CloseDropDownMenus()
									end,
								},
								showallopts = {
									type = "toggle",
									order = 40,
									width = "full",
									name = L["Show the tooltip icon, coloring, and count settings from below on the Right-Click menu."],
									descStyle = "inline",
									desc = "|cffff0000"..L["Enabling and using these options on any module, will affect all of them."].."|r",
									get = function() return core.db.profile.showallopts end,
									set = function(info, value)
										core.db.profile.showallopts = value
										CloseDropDownMenus()
									end,
								},
							},
						},
						fonts = {
							type = "group",
							name = L["Font Options"],
							childGroups = "tab",
							order = 45,
							args = {
								space2 = {
									order = 47,
									type = "description",
									name = L["The following settings apply to all tooltips."],
								},
								space3 = {
									order = 48,
									type = "description",
									name = " "
								},
								name = {
									type = "group",
									name = L["Name Font"],
									inline = true,
									order = 50,
									args = {

										namefont = {
											name = L["Font"],
											desc = L["The font used for this element."],
											type = "select", dialogControl = 'LSM30_Font',
											order = 55,
											values = AceGUIWidgetLSMlists.font,
											get = function() return core.db.profile.namefont end,
											set = function(info, value)
												core.db.profile.namefont = value
												core:updateFonts()
											end,
										},
										namefontol = {
											name = L["Font Outline"],
											desc = L["The outline that this font will use."],
											type = "select",
											order = 60,
											values = outlines,
											get = function() return core.db.profile.namefontol end,
											set = function(info, value)
												core.db.profile.namefontol = value
												core:updateFonts()
											end,
										},
										namefontsize = {
											name = L["Font Size"],
											desc = L["The size that this font will use."],
											type = "range",
											order = 65,
											min = 8,
											max = 24,
											step = 1,
											get = function() return core.db.profile.namefontsize end,
											set = function(info, value)
												core.db.profile.namefontsize = value
												core:updateFonts()
											end,
										},
									},
								},
								header = {
									type = "group",
									name = L["Header Font"],
									inline = true,
									order = 70,
									args = {
										headerfont = {
											name = L["Font"],
											desc = L["The font used for this element."],
											type = "select", dialogControl = 'LSM30_Font',
											order = 75,
											values = AceGUIWidgetLSMlists.font,
											get = function() return core.db.profile.headerfont end,
											set = function(info, value)
												core.db.profile.headerfont = value
												core:updateFonts()
											end,
										},
										headerfontol = {
											name = L["Font Outline"],
											desc = L["The outline that this font will use."],
											type = "select",
											order = 80,
											values = outlines,
											get = function() return core.db.profile.headerfontol end,
											set = function(info, value)
												core.db.profile.headerfontol = value
												core:updateFonts()
											end,
										},
										headerfontsize = {
											name = L["Font Size"],
											desc = L["The size that this font will use."],
											type = "range",
											order = 85,
											min = 8,
											max = 24,
											step = 1,
											get = function() return core.db.profile.headerfontsize end,
											set = function(info, value)
												core.db.profile.headerfontsize = value
												core:updateFonts()
											end,
										},
									},
								},
								text = {
									type = "group",
									name = L["Text Font"],
									inline = true,
									order = 90,
									args = {
										textfont = {
											name = L["Font"],
											desc = L["The font used for this element."],
											type = "select", dialogControl = 'LSM30_Font',
											order = 95,
											values = AceGUIWidgetLSMlists.font,
											get = function() return core.db.profile.textfont end,
											set = function(info, value)
												core.db.profile.textfont = value
												core:updateFonts()
											end,
										},
										textfontol = {
											name = L["Font Outline"],
											desc = L["The outline that this font will use."],
											type = "select",
											order = 100,
											values = outlines,
											get = function() return core.db.profile.textfontol end,
											set = function(info, value)
												core.db.profile.textfontol = value
												core:updateFonts()
											end,
										},
										textfontsize = {
											name = L["Font Size"],
											desc = L["The size that this font will use."],
											type = "range",
											order = 105,
											min = 8,
											max = 24,
											step = 1,
											get = function() return core.db.profile.textfontsize end,
											set = function(info, value)
												core.db.profile.textfontsize = value
												core:updateFonts()
											end,
										},
									},
								},
								number = {
									type = "group",
									name = L["Number Font"],
									inline = true,
									order = 110,
									args = {
										numberfont = {
											name = L["Font"],
											desc = L["The font used for this element."],
											type = "select", dialogControl = 'LSM30_Font',
											order = 115,
											values = AceGUIWidgetLSMlists.font,
											get = function() return core.db.profile.numberfont end,
											set = function(info, value)
												core.db.profile.numberfont = value
												core:updateFonts()
											end,
										},
										numberfontol = {
											name = L["Font Outline"],
											desc = L["The outline that this font will use."],
											type = "select",
											order = 120,
											values = outlines,
											get = function() return core.db.profile.numberfontol end,
											set = function(info, value)
												core.db.profile.numberfontol = value
												core:updateFonts()
											end,
										},
										numberfontsize = {
											name = L["Font Size"],
											desc = L["The size that this font will use."],
											type = "range",
											order = 125,
											min = 8,
											max = 24,
											step = 1,
											get = function() return core.db.profile.numberfontsize end,
											set = function(info, value)
												core.db.profile.numberfontsize = value
												core:updateFonts()
											end,
										},
									},
								},
							},
						},
						tooltip = {
							type = "group",
							name = L["Tooltip Options"],
							childGroups = "tab",
							order = 130,
							args = {
								space4 = {
									order = 131,
									type = "description",
									name = L["The following settings apply to all tooltips."],
								},
								space5 = {
									order = 133,
									type = "description",
									name = " "
								},
								scale = {
									order = 135,
									type = "range",
									name = L["Scale"],
									min = .5,
									max = 1.5,
									step = .05,
								},
								scroll = {
									order = 140,
									type = "range",
									name = L["Height"],
									min = 100,
									max = 900,
									step = 25,
								},
								tooltipIcons = {
									type = "toggle",
									order = 145,
									width = "full",
									name = L["Show icons in the tooltip."],
								},
								tooltipIconsOnly = {
									type = "toggle",
									order = 150,
									width = "full",
									name = L["Show icons only in the tooltip."],
									hidden = function() return not core.db.profile.tooltipIcons end
								},
								tooltipShowShort = {
									type = "toggle",
									order = 155,
									width = "full",
									name = L["Show short text in the tooltip."],
									hidden = function() if core.db.profile.tooltipIcons and core.db.profile.tooltipIconsOnly then return true end end
								},
								lineColor = {
									type = "toggle",
									order = 160,
									width = "full",
									name = L["Color lines in the tooltip."],
								},
								textColor = {
									type = "toggle",
									order = 165,
									width = "full",
									name = L["Color text in the tooltip."],
								},
								showZero = {
									type = "toggle",
									order = 170,
									width = "full",
									name = L["Show zero counts in the tooltip."],
								},
								showBank = {
									type = "toggle",
									order = 175,
									width = "full",
									name = L["Show bank counts in the tooltip."],
								},
								showGuild = {
									type = "toggle",
									order = 180,
									width = "full",
									name = L["Show guild counts in the tooltip."],
								},
								showAlts = {
									type = "toggle",
									order = 185,
									width = "full",
									name = L["Show alt counts in the tooltip."],
								},
								showTotal = {
									type = "toggle",
									order = 190,
									width = "full",
									name = L["Show total count from all characters and sources in the tooltip."],
								},
							},
						},
						tracking = {
							type = "group",
							name = L["Tracker Options"],
							childGroups = "tab",
							order = 195,
							args = {
								space6 = {
									order = 131,
									type = "description",
									name = L["The following settings apply to all trackers."],
								},
								space7 = {
									order = 133,
									type = "description",
									name = " "
								},
								showIcons = {
									type = "toggle",
									order = 200,
									width = "full",
									name = L["Show icons on the tracker."],
									get = function() return core.db.profile.showIcons end,
									set = function(info, value)
										core.db.profile.showIcons = value
										core:applySettings()
									end,
								},
								iconsOnly = {
									type = "toggle",
									order = 205,
									width = "full",
									name = L["Show icons only on the tracker."],
									get = function() return core.db.profile.iconsOnly end,
									set = function(info, value)
										core.db.profile.iconsOnly = value
										core:applySettings()
									end,
									hidden = function() return not core.db.profile.showIcons end
								},
								showShort = {
									type = "toggle",
									order = 210,
									width = "full",
									name = L["Show short text on the tracker."],
									get = function() return core.db.profile.showShort end,
									set = function(info, value)
										core.db.profile.showShort = value
										core:applySettings()
									end,
									hidden = function() if core.db.profile.showIcons and core.db.profile.iconsOnly then return true end end
								},
								trackerColor = {
									type = "toggle",
									order = 215,
									width = "full",
									name = L["Color text in the tracker."],
									get = function() return core.db.profile.trackerColor end,
									set = function(info, value)
										core.db.profile.trackerColor = value
										core:applySettings()
									end,
								},
							},
						},
						purge = {
							type = "group",
							name = L["Delete Character"],
							childGroups = "tab",
							order = 220,
							args = {
								deleteChar = {
									type = "select",
									order = 225,
									name = L["Delete Character"],
									desc = "|cffff0000"..L["This action will force a UI reload."].."|r",
									values = private.charsList,
									set = function(v, alt)
										core:purgeData(alt)
									end
								},
							},
						},
					},
				},
			},
		}
		for k,v in pairs(moduleOptions) do
			options.args[k] = (type(v) == "function") and v() or v
		end
	end
	return options
end

function core:setupOptions()
	self.optionsFrames = {}
	LS("AceConfigRegistry-3.0"):RegisterOptionsTable(FOLDER_NAME, getOptions)
	self.optionsFrames.core = LS("AceConfigDialog-3.0"):AddToBlizOptions(FOLDER_NAME, nil, nil, "general")
	self:registerModuleOptions("Profiles", LS("AceDBOptions-3.0"):GetOptionsTable(self.db), "Profiles")
end

function core:registerModuleOptions(name, optionTbl, displayName)
	moduleOptions[name] = optionTbl
	self.optionsFrames[name] = LS("AceConfigDialog-3.0"):AddToBlizOptions(FOLDER_NAME, displayName, FOLDER_NAME, name)
end

function core:purgeData(alt)
	core.db.realm.chars[alt] = nil
	ReloadUI()
end
